import json, os
from tabulate import tabulate

localizationKey = "localization"

# Verifies missing keys for the group of prompts and reasons in 20x4, 40x2 and Vipa for English only json files
#
# Limitiations: Script do not verifies other languages than en_EN
# TODO: Add verifications for all the languages files 

class MissingKeysVerification:
	# private method that verifies group of files looking for discrepencies in keys
	def verifyFiles(self, listOfFiles, workingDir, result): 
		localizations = []
		totalDifferencesAmount = 0

		for fileName in listOfFiles:
			filePath = os.path.join(workingDir, fileName)
			with open(filePath) as f:
				jsonData = json.load(f)
				localizationKeys = jsonData[localizationKey].keys()
				localizations.append([fileName, localizationKeys])

		i = 0
		while i < len(localizations) - 1:
			dataSet1 = localizations[i][1]
			dataSet2 = localizations[i+1][1]
			differenceSet = set(dataSet1).difference(dataSet2)
		
			if len(differenceSet) > 0:
				for missingKey in differenceSet:
					fileName = ""
					if missingKey in dataSet1:
						fileName = localizations[i][0]
					else:
						fileName = localizations[i+1][0]
					result.append([fileName, missingKey])
	
			i += 1

	# Public verify method that fire missing keys verification for the group of files
	def verify(self, workingDir):
		inputFilesGroup = [
["displays/20x4/reasons-20x4-en_US.json", "displays/40x2/prompts-40x2-en_US.json", "displays/vipa/prompts-vipa-en_US.json"],
["displays/20x4/reasons-20x4-en_US.json", "displays/40x2/reasons-40x2-en_US.json", "displays/vipa/reasons-vipa-en_US.json"]]

		print("Verification missing keys started ..")
		result = []

		for promptsFileNames in inputFilesGroup:
			self.verifyFiles(promptsFileNames, workingDir, result)

		if len(result) == 0: 
			print("No missing keys found.")
		else:
			print("Found " + `len(result)` + " issues. Please fix the discrepencies in the Google spreadsheet files first and regenerate the prompt files again.")

			def sortList(e):
				return e[0] + e[1]
			result.sort(key=sortList)
			print tabulate(result, headers=['File', 'Key'])
		
		print("Verification missing keys finished.")



