import json, os
from tabulate import tabulate

localizationKey = "localization"

# Verifies prompt length (each line of the prompt) in according to max length and the count of available rows.
# Limitiations: Does not verify Vipa displays file and errors.
class PromptLengthVerification:
	# verifies length of the each line of prompt using @length and @expectedLinesCount
	# the fault results are stored in @resultPromptLengthMap and @resultPromptLinesCountMap
	def verifyLength(self, file, length, expectedLinesCount, resultPromptLengthMap, resultPromptLinesCountMap):
		with open(file) as f:
			jsonData = json.load(f)
			map = jsonData[localizationKey]

		for key in map:
			value = map[key]
			actualLinesCount = len(value)
			fileName = os.path.basename(file)

			if actualLinesCount != expectedLinesCount:
				message = [fileName, key, actualLinesCount, expectedLinesCount]
				resultPromptLinesCountMap.append(message)
				
			for line in value:
				line = line.encode("ascii", "replace")
				if len(line) > length:
					message = [fileName, key, line]
					resultPromptLengthMap.append(message)

	# executes verification for 20x4 and 40x2 displays and
	# prints the result
	def verify(self, workingDir):
		resultPromptLengthMap = []
		resultPromptLinesCountMap = []

		def sortFunc(e):
			return e[0] + e[1]


		print("Verification prompts length started ..")
		
		fileLengthMap = {"displays/20x4": [20, 4, 2], "displays/40x2": [40, 2, 1]}

		for dirName in fileLengthMap:
			dirPath = os.path.join(workingDir, dirName)
			for translationFile in os.listdir(dirPath):
				if translationFile.endswith(".json"):
					maxLength = fileLengthMap[dirName][0]
					promptsLinesCount = fileLengthMap[dirName][1]
					reasonsLinesCount = fileLengthMap[dirName][2]
					translationFilePath = os.path.join(dirPath, translationFile)

					if 'prompts' in translationFile:
						linesCount = promptsLinesCount
					elif 'reasons' in translationFile:
						linesCount = reasonsLinesCount

					self.verifyLength(translationFilePath, maxLength, linesCount, resultPromptLengthMap, resultPromptLinesCountMap)
	
		if len(resultPromptLengthMap) > 0:
			print("\nFailed: " + `len(resultPromptLengthMap)` + " prompts has wrong prompts length in line, the wrong prompts list:")
			resultPromptLengthMap.sort(key=sortFunc)
			print tabulate(resultPromptLengthMap, headers = ['File', 'Key', 'Value'])

		if len(resultPromptLinesCountMap) > 0:
			print("\nFailed: " + `len(resultPromptLinesCountMap)` + " prompts has wrong amount of lines:")

				
			resultPromptLinesCountMap.sort(key=sortFunc)
			print tabulate(resultPromptLinesCountMap, headers = ['File', 'Key', 'Actual', 'Expected'])

		print("Verification prompts length finished")
